@@ -42,7 +42,7 @@ global_client_free (void)
 		return;
 	}
 	
-	gtk_object_unref (GTK_OBJECT (global_gconf_client));
+	g_object_unref (global_gconf_client);
 	global_gconf_client = NULL;
 }
 
@@ -321,6 +321,28 @@ eel_gconf_monitor_remove (const char *directory)
 	return TRUE;
 }
 
+void
+eel_gconf_preload_cache (const char             *directory,
+			 GConfClientPreloadType  preload_type)
+{
+	GError *error = NULL;
+	GConfClient *client;
+
+	if (directory == NULL) {
+		return;
+	}
+
+	client = gconf_client_get_default ();
+	g_return_if_fail (client != NULL);
+	
+	gconf_client_preload (client,
+			      directory,
+			      preload_type,
+			      &error);
+	
+	eel_gconf_handle_error (&error);
+}
+
 void
 eel_gconf_suggest_sync (void)
 {
@@ -358,6 +380,30 @@ eel_gconf_get_value (const char *key)
 	return value;
 }
 
+GConfValue*
+eel_gconf_get_default_value (const char *key)
+{
+	GConfValue *value = NULL;
+	GConfClient *client;
+	GError *error = NULL;
+	
+	g_return_val_if_fail (key != NULL, NULL);
+
+	client = eel_gconf_client_get_global ();
+	g_return_val_if_fail (client != NULL, NULL);
+
+	value = gconf_client_get_default_from_schema (client, key, &error);
+	
+	if (eel_gconf_handle_error (&error)) {
+		if (value != NULL) {
+			gconf_value_free (value);
+			value = NULL;
+		}
+	}
+
+	return value;
+}
+
 static gboolean
 simple_value_is_equal (const GConfValue *a,
 		       const GConfValue *b)
@@ -562,3 +608,21 @@ eel_gconf_value_set_string_list (GConfValue *value,
 	}
 	g_slist_free (value_list);
 }
+
+EelStringList *
+eel_gconf_value_get_eel_string_list (const GConfValue *value)
+{
+	GSList *slist;
+	EelStringList *result;
+
+	if (value == NULL) {
+		return eel_string_list_new (TRUE);
+	}
+	g_return_val_if_fail (value->type == GCONF_VALUE_LIST, eel_string_list_new (TRUE));
+	g_return_val_if_fail (gconf_value_get_list_type (value) == GCONF_VALUE_STRING, eel_string_list_new (TRUE));
+
+	slist = eel_gconf_value_get_string_list (value);
+	result = eel_string_list_new_from_g_slist (slist, TRUE);
+	eel_g_slist_free_deep (slist);
+ 	return result;
+}
