@@ -31,11 +31,12 @@
 #include <eel/eel-gtk-macros.h>
 #include <eel/eel-string.h>
 #include <gtk/gtkclist.h>
+#include <gtk/gtkdialog.h>
 #include <gtk/gtknotebook.h>
+#include <gtk/gtkstock.h>
 #include <eel/eel-gtk-extensions.h>
 #include <libgnome/gnome-i18n.h>
 #include <libgnome/gnome-util.h>
-#include <libgnomeui/gnome-stock.h>
 
 #define NUM_CATEGORY_COLUMNS 1
 #define CATEGORY_COLUMN 0
@@ -58,11 +59,11 @@ struct EelPreferencesBoxDetails
 };
 
 /* EelPreferencesBoxClass methods */
-static void      eel_preferences_box_initialize_class   (EelPreferencesBoxClass *preferences_box_class);
-static void      eel_preferences_box_initialize         (EelPreferencesBox      *preferences_box);
+static void      eel_preferences_box_class_init   (EelPreferencesBoxClass *preferences_box_class);
+static void      eel_preferences_box_init         (EelPreferencesBox      *preferences_box);
 
-/* GtkObjectClass methods */
-static void      eel_preferences_box_destroy            (GtkObject              *object);
+/* GObjectClass methods */
+static void      eel_preferences_box_finalize           (GObject                *object);
 
 /* Misc private stuff */
 static void      preferences_box_category_list_recreate (EelPreferencesBox      *preferences_box);
@@ -85,38 +86,33 @@ static int       preferences_box_find_row               (GtkCList
 							 char                   *pane_name);
 static void      user_level_changed_callback            (gpointer                callback_data);
 
-EEL_DEFINE_CLASS_BOILERPLATE (EelPreferencesBox, eel_preferences_box, GTK_TYPE_HBOX)
+EEL_CLASS_BOILERPLATE (EelPreferencesBox, eel_preferences_box, GTK_TYPE_HBOX)
 
 /*
  * EelPreferencesBoxClass methods
  */
 static void
-eel_preferences_box_initialize_class (EelPreferencesBoxClass *preferences_box_class)
+eel_preferences_box_class_init (EelPreferencesBoxClass *preferences_box_class)
 {
-	GtkObjectClass *object_class;
-	
-	object_class = GTK_OBJECT_CLASS (preferences_box_class);
-
-	/* GtkObjectClass */
-	object_class->destroy = eel_preferences_box_destroy;
+	G_OBJECT_CLASS (preferences_box_class)->finalize = eel_preferences_box_finalize;
 }
 
 static void
-eel_preferences_box_initialize (EelPreferencesBox *preferences_box)
+eel_preferences_box_init (EelPreferencesBox *preferences_box)
 {
 	preferences_box->details = g_new0 (EelPreferencesBoxDetails, 1);
 
 	eel_preferences_add_callback_while_alive ("user_level",
 						  user_level_changed_callback,
 						  preferences_box,
-						  GTK_OBJECT (preferences_box));
+						  G_OBJECT (preferences_box));
 }
 
 /*
- * GtkObjectClass methods
+ * GObjectClass methods
  */
 static void
-eel_preferences_box_destroy (GtkObject *object)
+eel_preferences_box_finalize (GObject *object)
 {
 	EelPreferencesBox *preferences_box;
 	
@@ -143,8 +139,7 @@ eel_preferences_box_destroy (GtkObject *object)
 	g_free (preferences_box->details->selected_pane);
 	g_free (preferences_box->details);
 	
-	/* Chain destroy */
-	EEL_CALL_PARENT (GTK_OBJECT_CLASS, destroy, (object));
+	EEL_CALL_PARENT (G_OBJECT_CLASS, finalize, (object));
 }
 
 /*
@@ -171,8 +166,9 @@ preferences_box_select_pane (EelPreferencesBox *preferences_box,
 		
 		if (eel_str_is_equal (pane_name, info->pane_name)) {
  			gtk_widget_show (GTK_WIDGET (info->pane_widget));
- 			gtk_notebook_set_page (GTK_NOTEBOOK (preferences_box->details->pane_notebook), 
- 					       g_list_position (preferences_box->details->panes, pane_iterator));
+ 			gtk_notebook_set_current_page
+				(GTK_NOTEBOOK (preferences_box->details->pane_notebook), 
+				 g_list_position (preferences_box->details->panes, pane_iterator));
 
 			g_free (preferences_box->details->selected_pane);
 			preferences_box->details->selected_pane = g_strdup (pane_name);
@@ -324,9 +320,9 @@ eel_preferences_box_new (void)
 	preferences_box->details->category_list = gtk_clist_new (NUM_CATEGORY_COLUMNS);
 
 	preferences_box->details->select_row_signal_id =
-		gtk_signal_connect (GTK_OBJECT (preferences_box->details->category_list), 
+		g_signal_connect (preferences_box->details->category_list, 
 				    "select_row",
-				    GTK_SIGNAL_FUNC (category_list_select_row_callback),
+				    G_CALLBACK (category_list_select_row_callback),
 				    preferences_box);
 
 	gtk_clist_set_selection_mode (GTK_CLIST (preferences_box->details->category_list), 
@@ -400,26 +396,42 @@ eel_preferences_box_update (EelPreferencesBox	*preferences_box)
 	preferences_box_category_list_recreate (preferences_box);
 }
 
-static EelPreferencesPane *
+static PaneInfo *
 preferences_box_find_pane (const EelPreferencesBox *preferences_box,
 			   const char *pane_name)
 {
 	GList *node;
 	PaneInfo *info;
 
-	g_return_val_if_fail (EEL_IS_PREFERENCES_BOX (preferences_box), FALSE);
+	g_return_val_if_fail (EEL_IS_PREFERENCES_BOX (preferences_box), NULL);
 
 	for (node = preferences_box->details->panes; node != NULL; node = node->next) {
 		g_assert (node->data != NULL);
 		info = node->data;
 		if (eel_str_is_equal (info->pane_name, pane_name)) {
-			return info->pane_widget;
+			return info;
 		}
 	}
 
 	return NULL;
 }
 
+static EelPreferencesPane *
+preferences_box_find_pane_widget (const EelPreferencesBox *preferences_box,
+				  const char *pane_name)
+{
+	PaneInfo *info;
+
+	g_return_val_if_fail (EEL_IS_PREFERENCES_BOX (preferences_box), NULL);
+
+	info = preferences_box_find_pane (preferences_box, pane_name);
+	if (info == NULL) {
+		return NULL;
+	}
+
+	return info->pane_widget;
+}
+
 static void
 preferences_box_populate_pane (EelPreferencesBox *preferences_box,
 			       const char *pane_name,
@@ -437,7 +449,7 @@ preferences_box_populate_pane (EelPreferencesBox *preferences_box,
 	g_return_if_fail (items != NULL);
 
 	/* Create the pane if needed */
-	pane = preferences_box_find_pane (preferences_box, pane_name);
+	pane = preferences_box_find_pane_widget (preferences_box, pane_name);
 	if (pane == NULL) {
 		pane = EEL_PREFERENCES_PANE (preferences_box_add_pane (preferences_box, pane_name));
 	}
@@ -445,7 +457,7 @@ preferences_box_populate_pane (EelPreferencesBox *preferences_box,
 	group_names = eel_string_list_new (TRUE);
 
 	for (i = 0; items[i].group_name != NULL; i++) {
-		translated_group_name = _(items[i].group_name);
+		translated_group_name = gettext (items[i].group_name);
 		if (!eel_string_list_contains (group_names, translated_group_name)) {
 			eel_string_list_insert (group_names, translated_group_name);
 			eel_preferences_pane_add_group (pane,
@@ -455,13 +467,13 @@ preferences_box_populate_pane (EelPreferencesBox *preferences_box,
 
 	for (i = 0; items[i].group_name != NULL; i++) {
 		group = EEL_PREFERENCES_GROUP (eel_preferences_pane_find_group (pane,
-										_(items[i].group_name)));
+										gettext (items[i].group_name)));
 		g_return_if_fail (EEL_IS_PREFERENCES_GROUP (group));
 
 		if (items[i].preference_name != NULL) {
 			if (items[i].preference_description != NULL) {
 				eel_preferences_set_description (items[i].preference_name,
-								 _(items[i].preference_description));
+								 gettext (items[i].preference_description));
 			}
 		
 			item = EEL_PREFERENCES_ITEM (eel_preferences_group_add_item (group,
@@ -508,7 +520,7 @@ eel_preferences_box_populate (EelPreferencesBox *preferences_box,
 
 	for (i = 0; panes[i].pane_name != NULL; i++) {
 		preferences_box_populate_pane (preferences_box,
-					       _(panes[i].pane_name),
+					       gettext (panes[i].pane_name),
 					       panes[i].items);
 	}
 
@@ -527,34 +539,28 @@ GtkWidget *
 eel_preferences_dialog_new (const char *title,
 			    const EelPreferencesPaneDescription *panes)
 {
-	static const char *stock_buttons[] = {
-		GNOME_STOCK_BUTTON_OK,
-		NULL
-	};
 	GtkWidget *dialog;
 	GtkWidget *preference_box;
 	GtkWidget *vbox;
 
 	g_return_val_if_fail (title != NULL, NULL);
-	g_return_val_if_fail (panes != NULL, NULL);
 	
-	dialog = gnome_dialog_newv (title, stock_buttons);
+	dialog = gtk_dialog_new_with_buttons (title, NULL, 0,
+					      GTK_STOCK_OK, GTK_RESPONSE_OK,
+					      NULL);
 
 	/* Setup the dialog */
-	gtk_window_set_policy (GTK_WINDOW (dialog), 
-			       FALSE,	/* allow_shrink */
-			       TRUE,	/* allow_grow */
-			       FALSE);	/* auto_shrink */
+	gtk_window_set_resizable (GTK_WINDOW (dialog), TRUE);
 
   	gtk_container_set_border_width (GTK_CONTAINER (dialog), 0);
 	
-	gnome_dialog_set_default (GNOME_DIALOG(dialog), 0);
+	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
 
 	eel_gtk_window_set_up_close_accelerator (GTK_WINDOW (dialog));
 
 	preference_box = eel_preferences_box_new ();
 
-	vbox = GNOME_DIALOG (dialog)->vbox;
+	vbox = GTK_DIALOG (dialog)->vbox;
 	
 	gtk_box_set_spacing (GTK_BOX (vbox), 10);
 	
@@ -566,8 +572,115 @@ eel_preferences_dialog_new (const char *title,
 
 	gtk_widget_show (preference_box);
 
-	eel_preferences_box_populate (EEL_PREFERENCES_BOX (preference_box),
-				      panes);
+	if (panes != NULL) {
+		eel_preferences_dialog_populate (GTK_WINDOW (dialog), panes);
+	}
 	
 	return dialog;
 }
+
+EelPreferencesBox *
+eel_preferences_dialog_get_box (const GtkWindow *dialog)
+{
+	GtkWidget *vbox;
+	const GList *last_node;
+	const GtkBoxChild *box_child;
+
+	g_return_val_if_fail (GTK_IS_DIALOG (dialog), NULL);
+
+	vbox = GTK_DIALOG (dialog)->vbox;
+
+	g_return_val_if_fail (GTK_IS_VBOX (vbox), NULL);
+	last_node = g_list_last (GTK_BOX (vbox)->children);
+	g_return_val_if_fail (last_node != NULL, NULL);
+	g_return_val_if_fail (last_node->data != NULL, NULL);
+	box_child = last_node->data;
+	g_return_val_if_fail (EEL_IS_PREFERENCES_BOX (box_child->widget), NULL);
+	return EEL_PREFERENCES_BOX (box_child->widget);
+}
+
+void
+eel_preferences_dialog_populate (GtkWindow *dialog,
+				 const EelPreferencesPaneDescription *panes)
+{
+	EelPreferencesBox *preferences_box;
+
+	g_return_if_fail (GTK_IS_WINDOW (dialog));
+	g_return_if_fail (panes != NULL);
+	preferences_box = eel_preferences_dialog_get_box (dialog);
+	g_return_if_fail (EEL_IS_PREFERENCES_BOX (preferences_box));
+	eel_preferences_box_populate (preferences_box, panes);
+}
+
+void
+eel_preferences_box_for_each_pane (const EelPreferencesBox *preferences_box,
+				   EelPreferencesBoxForEachCallback callback,
+				   gpointer callback_data)
+{
+	GList *node;
+	PaneInfo *info;
+	
+	g_return_if_fail (EEL_IS_PREFERENCES_BOX (preferences_box));
+	g_return_if_fail (callback != NULL);
+	
+	for (node = preferences_box->details->panes; node != NULL; node = node->next) {
+		g_assert (node->data != NULL);
+		info = node->data;
+		(* callback) (info->pane_name, info->pane_widget, callback_data);
+	}
+}
+
+void
+eel_preferences_box_rename_pane (EelPreferencesBox *preferences_box,
+				 const char *pane_name,
+				 const char *new_pane_name)
+{
+	PaneInfo *pane_info;
+	int row;
+
+	g_return_if_fail (EEL_IS_PREFERENCES_BOX (preferences_box));
+	g_return_if_fail (eel_strlen (pane_name) > 0);
+	g_return_if_fail (eel_strlen (new_pane_name) > 0);
+
+	if (eel_str_is_equal (pane_name, new_pane_name)) {
+		return;
+	}
+
+	pane_info = preferences_box_find_pane (preferences_box, pane_name);
+	if (pane_info == NULL) {
+		g_warning ("The box does not have a pane called '%s'", pane_name);
+		return;
+	}
+
+	row = preferences_box_find_row (GTK_CLIST (preferences_box->details->category_list),
+					pane_info->pane_name);
+	g_assert (row != -1);
+
+	g_free (pane_info->pane_name);
+	pane_info->pane_name = g_strdup (new_pane_name);
+	gtk_clist_set_text (GTK_CLIST (preferences_box->details->category_list),
+			    row,
+			    CATEGORY_COLUMN,
+			    pane_info->pane_name);
+}
+
+char *
+eel_preferences_box_get_pane_name (const EelPreferencesBox *preferences_box,
+				   const EelPreferencesPane *pane)
+{
+	GList *node;
+	PaneInfo *info;
+	
+	g_return_val_if_fail (EEL_IS_PREFERENCES_BOX (preferences_box), NULL);
+	g_return_val_if_fail (EEL_IS_PREFERENCES_PANE (pane), NULL);
+	
+	for (node = preferences_box->details->panes; node != NULL; node = node->next) {
+		g_assert (node->data != NULL);
+		info = node->data;
+		if (info->pane_widget == pane) {
+			return g_strdup (info->pane_name);
+		}
+	}
+
+	return NULL;
+}
