@@ -5,15 +5,15 @@
 
 #include <gtk/gtksignal.h>
 
-static void eel_dumb_box_initialize_class (EelDumbBoxClass *dumb_box_class);
-static void eel_dumb_box_initialize       (EelDumbBox      *dumb_box);
+static void eel_dumb_box_class_init (EelDumbBoxClass *dumb_box_class);
+static void eel_dumb_box_init       (EelDumbBox      *dumb_box);
 static gint eel_dumb_box_expose           (GtkWidget       *widget,
 					   GdkEventExpose  *event);
 
-EEL_DEFINE_CLASS_BOILERPLATE (EelDumbBox, eel_dumb_box, GTK_TYPE_BIN)
+EEL_CLASS_BOILERPLATE (EelDumbBox, eel_dumb_box, GTK_TYPE_BIN)
 
 static void
-eel_dumb_box_initialize_class (EelDumbBoxClass *dumb_box_class)
+eel_dumb_box_class_init (EelDumbBoxClass *dumb_box_class)
 {
 	GtkWidgetClass *widget_class;
 	
@@ -26,7 +26,7 @@ eel_dumb_box_initialize_class (EelDumbBoxClass *dumb_box_class)
 }
 
 static void
-eel_dumb_box_initialize (EelDumbBox *dumb_box)
+eel_dumb_box_init (EelDumbBox *dumb_box)
 {
 	GTK_WIDGET_UNSET_FLAGS (dumb_box, GTK_NO_WINDOW);
 }
@@ -42,18 +42,17 @@ eel_dumb_box_expose (GtkWidget *widget,
 		     GdkEventExpose *event)
 {
 	GtkBin *bin;
-	GdkEventExpose child_event;
 
 	g_return_val_if_fail (GTK_IS_DUMB_BOX (widget), FALSE);
 	g_return_val_if_fail (event != NULL, FALSE);
 
 	bin = GTK_BIN (widget);
-	
-	child_event = *event;
-	if (bin->child && GTK_WIDGET_NO_WINDOW (bin->child)
-	    && gtk_widget_intersect (bin->child, &event->area, &child_event.area)) {
-		gtk_widget_event (bin->child, (GdkEvent*) &child_event);
+
+	if (bin->child != NULL) {
+		gtk_container_propagate_expose (GTK_CONTAINER (widget),
+						bin->child,
+						event);
 	}
-	
+
 	return FALSE;
 }
